@@ -31,17 +31,21 @@ import re
 import subprocess
 import os
 import tempfile
+import sys
 
 
 def has_ca():
     try:
-        if subprocess.mswindows:
+        if sys.platform == 'win32':
             startupinfo = subprocess.STARTUPINFO()
             startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
             p = subprocess.Popen(
-                ['certutil', '-ping'], stdout=subprocess.PIPE,
+                ['certutil', '-dump'], stdout=subprocess.PIPE,
                 startupinfo=startupinfo)
-            return p.returncode == 0
+            output, _ = p.communicate()
+            # 'certutil -dump' returns one line when no CA is available.
+            return len(str.splitlines(output)) > 1
+
     except OSError:
         pass
     return False
@@ -139,3 +143,11 @@ def b2len(bs):
         l *= 256
         l += byte2int(b)
     return l
+
+
+def is_macos_sierra_or_later():
+    if sys.platform == 'darwin':
+        from platform import mac_ver
+        mac_version = tuple(int(x) for x in mac_ver()[0].split('.'))
+        return mac_version >= (10, 12)
+    return False
